package test;

/**
 * Class representing a structure on a linear DNA molecule and the things
 * that can happen to it. A structure has a name (e.g. gene1) and position. 
 * It can also be on (working) or off (disrupted).
 * A structure is off when the ends of the linear DNA molecule on which it is
 * comtained extend beyond the structure's start or end positions.
 * The knock-on effects of this are that the quantities and rates of any
 * reactions in which the structure participates must be changed.
 * @author R J Crossland
 */
public class Structure {

    String[] names;
    double[] startBps;
    double[] endBps;
    boolean[] onOrOffs;
    double[][] offK;
    int offX;

    /**
     * Constructor for a structure.
     * @param name  structure name.
     * @param startBp  starting position of structure on linear DNA.
     * @param endBP  end position of structure on linear DNA.
     * @param onOrOff  whether structure is working or disrupted.
     * @param n  total number of time points in simulation.
     * @param disruptK  changes in reaction rates as a result of disruption.
     * @param disruptX  changes in quantities as a result of disruption.
     */
    public Structure(String name, int startBp, int endBp, boolean onOrOff,
            double n, double[][] disruptK, int disruptX) {

        this.names = new String[(int) n];
        this.startBps = new double[(int) n];
        this.endBps = new double[(int) n];
        this.onOrOffs = new boolean[(int) n];
        this.offK = disruptK;
        this.offX = disruptX;

        for (int i = 0; i < onOrOffs.length; i++) { // fill with default values

            this.names[i] = name;
            this.startBps[i] = startBp;
            this.endBps[i] = endBp;
            this.onOrOffs[i] = onOrOff;
        }
    }

    /**
     * The knock-on effects on quantities and rates caused by a disrupted
     * structure on the linear DNA.
     * @param cell  cellular compartment in which reactions occur.
     * @param i  current time point in simulation.
     * @param dna1  the linear DNA molecule of which structure is a part.
     */
    public void disruptStructure(Cell cell, int i, LinearDNA dna1) {

        if (dna1.lengths[i] < this.endBps[i]) {

            this.onOrOffs[i] = false;
            cell.allX[i][this.offX] = 0; // change quantity of structure

            //change rate constants as a result of disrupted structure
            for (int allKRow = 0; allKRow < this.offK.length; allKRow++) {

                double reaction = this.offK[allKRow][0];
                double rate = this.offK[allKRow][1];

                cell.allK[i][(int) reaction] = rate;
            }
        } else {
            this.onOrOffs[i] = true;
        }
    }
}
